@@ -35,7 +35,7 @@
 #include <linux/init.h>
 #include <linux/ioport.h>
 #include <linux/proc_fs.h>
-#include <linux/version.h>
+#include <linux/devfs_fs_kernel.h>
 #include <asm/uaccess.h>
 #include <asm/io.h>
 #include "thinkpad_common.h"
@@ -49,11 +49,27 @@
 
 extern struct proc_dir_entry *thinkpad_ppde;
 
+#ifdef CONFIG_DEVFS_FS
+static int superio_open(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+);
+static int superio_release(
+	struct inode * pinodeThe,
+	struct file * pfileThe 
+);
+static int superio_ioctl(
+	struct inode * pinodeThe,
+	struct file * pfileThe,
+	unsigned int uintIoctlNum,
+	unsigned long ulongIoctlArg
+);
+#endif
 
 /****** variables ******/
 static const char _szMyName[] = "superio";
 static const char _szImName[] = "superio_do";
-static const char _szMyVersion[] = "5.0";
+static const char _szMyVersion[] = "4.0";
 static const char _szProcfile[] = "driver/thinkpad/superio";
 static word _wPortIndex, _wPortData;
 static struct resource *_presourceSuperio;
@@ -61,9 +77,21 @@ static struct resource *_presourceSuperio;
 #ifdef MODULE
 MODULE_AUTHOR( "Thomas Hood" );
 MODULE_DESCRIPTION( "Driver for the Super I/O chip on IBM ThinkPads" );
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10)
 MODULE_LICENSE( "GPL" );
 #endif
+#endif
 
+#ifdef CONFIG_DEVFS_FS
+static struct file_operations _fileopsSuperio = {
+	.ioctl   = superio_ioctl,
+	.open    = superio_open,
+	.release = superio_release,
+	.owner   = THIS_MODULE
+};
+
+static devfs_handle_t _devfs_handleSuperio;
+#endif
 
 /****** functions ******/
 
@@ -77,10 +105,10 @@ static byte getregb( byte bWhere )
 	byte bGot;
 	unsigned long flags;
 
-	local_irq_save( flags );
+	save_flags( flags ); cli();
 	outb_p( bWhere, _wPortIndex );
 	bGot = inb( _wPortData );
-	local_irq_restore( flags );
+	restore_flags( flags );
 
 	return bGot;
 }
@@ -90,10 +118,10 @@ static void putregb( byte bWhat, byte bWhere )
 {
 	unsigned long flags;
 
-	local_irq_save( flags );
+	save_flags( flags ); cli();
 	outb_p( bWhere, _wPortIndex );
 	outb( bWhat, _wPortData );
-	local_irq_restore( flags );
+	restore_flags( flags );
 
 	return;
 }
@@ -195,10 +223,10 @@ static flag_t is_in_pnp_mode( word wPortIndex, word wPortData )
 	byte bGot;
 	unsigned long flags;
 
-	local_irq_save( flags );
+	save_flags( flags ); cli();
 	outb_p( 0x1b, wPortIndex );
 	bGot = inb( wPortData );
-	local_irq_restore( flags );
+	restore_flags( flags );
 
 	return (flag_t)( (bGot >> 3) & 1 );
 }
@@ -754,6 +782,61 @@ int superio_do(
 	return intRtnDo; /* Should be either 0 or -EACCES or -EINVAL */
 }
 
+#ifdef CONFIG_DEVFS_FS
+static int superio_open(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+) {
+
+	return 0;
+}
+
+
+static int superio_release(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+) {
+
+	return 0;
+}
+
+
+static flag_t caller_has_w( struct file * pfileThe )
+{
+	return ((pfileThe->f_mode) & FMODE_WRITE) ? 1 : 0;
+}
+
+
+static int superio_ioctl(
+	struct inode * pinodeThe,
+	struct file * pfileThe,
+	unsigned int uintIoctlNum,
+	unsigned long ulongIoctlArg
+) {
+
+#ifdef DEBUG_VERBOSE
+	printk( "%s: Doing ioctl number 0x%x\n", _szMyName, uintIoctlNum );
+#endif
+
+	switch ( uintIoctlNum ) {
+		/* should  return  at the end of each case block */
+
+		case IOCTL_SUPERIO_REQUEST: 
+			return superio_do(
+				ulongIoctlArg, 
+				caller_has_w( pfileThe )
+			);
+
+		default:
+			/* ioctl number not recognized -- do nothing */
+			return -ENOTTY;
+
+	} /* switch */
+			
+	return -ETHINKPAD_PROGRAMMING; /* We should never arrive here */
+}
+#endif /* defined(CONFIG_DEVFS_FS) */
+
 
 /*
  * We write to nonsense port 0x80 in order to
@@ -765,7 +848,7 @@ static flag_t a_super_io_chip_appears_to_be_at( word wPort )
 	byte bSave, bID;
 	unsigned long flags;
 
-	local_irq_save( flags );
+	save_flags( flags ); cli();
 
 	bSave = inb( wPort );
 
@@ -789,12 +872,12 @@ static flag_t a_super_io_chip_appears_to_be_at( word wPort )
 
 /* Yes: */
 	/* No need to restore index port's contents */
-	local_irq_restore( flags );
+	restore_flags( flags );
 	return 1;
 
 No:
 	outb( bSave, wPort ); /* restore the port's contents */
-	local_irq_restore( flags );
+	restore_flags( flags );
 	return 0;
 }
 
@@ -887,6 +970,23 @@ static int __init superio_init( void )
 	}
 	/* proc entry created */
 
+#ifdef CONFIG_DEVFS_FS
+	_devfs_handleSuperio = devfs_register(
+		NULL /* dir */,
+		"thinkpad/superio",
+		DEVFS_FL_DEFAULT | DEVFS_FL_AUTO_DEVNUM,
+		0 /* major */, 0 /* minor */,
+		S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH,
+		&_fileopsSuperio,
+		NULL /* info */
+	);
+	if ( _devfs_handleSuperio == NULL ) {
+		printk(KERN_ERR "%s: Could not devfs_register(). Aborting.\n", _szMyName );
+		return -EIO;
+	}
+	/* devfs entry created */
+#endif
+
 	inter_module_register( _szImName, THIS_MODULE, &superio_do );
 
 	return 0;
@@ -898,6 +998,10 @@ static void __exit superio_exit( void )
 
 	inter_module_unregister( _szImName );
 
+#ifdef CONFIG_DEVFS_FS
+	devfs_unregister( _devfs_handleSuperio );
+#endif
+
  	remove_proc_entry( _szProcfile, NULL );
 
 	if ( _presourceSuperio != NULL ) release_resource( _presourceSuperio );
