@@ -10,7 +10,13 @@
  *
  * Notes:  This driver exists so that several thinkpad devices can be
  *         driven with only a single permanent misc device minor number
- *         allocated.
+ *         allocated.  With the advent of devfs, such tricks are no longer
+ *         necessary.  The elegant thing to do, under devfs, would be
+ *         for each driver to have its own device file under /dev/thinkpad.
+ *         However, the present scheme continues to work under devfs and
+ *         will not be changed unless there is a pressing need, since it
+ *         would involve substantial changes both to the drivers and to
+ *         the tpctl program that uses them.
  *
  * Please report bugs to the author ASAP.
  * 
@@ -34,8 +40,6 @@
 
 #include "thinkpad_driver.h"
 
-#define EXPORT_SYMTAB
-
 #include <linux/module.h>
 #include <linux/kmod.h>
 #include <linux/kernel.h>
@@ -43,9 +47,8 @@
 #include <linux/miscdevice.h>
 #include <linux/fs.h>
 #include <linux/proc_fs.h>
-#include <linux/version.h>
 #include <asm/uaccess.h>
-/* #include <linux/wrapper.h> */
+#include <linux/wrapper.h>
 #include "thinkpad_common.h"
 #include "thinkpad.h"
 #include "smapi.h"
@@ -100,12 +103,14 @@ MODULE_PARM( enable_rtcmosram, "i" );
 MODULE_PARM( enable_thinkpadpm, "i" );
 MODULE_AUTHOR( "Thomas Hood" );
 MODULE_DESCRIPTION( "Metadriver for IBM ThinkPad hardware drivers" );
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10)
 MODULE_LICENSE( "GPL" );
 #endif
+#endif
 
 /*** local ***/
 static const char _szMyName[] = "thinkpad";
-static const char _szMyVersion[] = "5.5";
+static const char _szMyVersion[] = "4.0";
 static const char _szSmapiName[] = "smapi";
 static const char _szSuperioName[] = "superio";
 static const char _szRtcmosramName[] = "rtcmosram";
@@ -300,7 +305,7 @@ static int thinkpad_ioctl(
 
 static struct miscdevice _structmiscdeviceThinkpad = {
 	.minor = MINOR_THINKPAD,
-	.name  = "thinkpad",
+	.name  = "thinkpad/thinkpad",
 	.fops  = &_fileopsThinkpad
 };
 
@@ -364,7 +369,6 @@ static void __exit thinkpad_exit( void )
 	return;
 }
 
-EXPORT_SYMBOL(thinkpad_ppde);
 module_init(thinkpad_init);
 module_exit(thinkpad_exit);
 
