@@ -81,13 +81,6 @@ MODULE_LICENSE( "GPL" );
  */
 #define APM_ZERO_SEGS
 
-/*
- * Save a segment register away
- */
-/* This is in include/asm/system.h */
-/* #define savesegment(seg, where) \ */
-/* 		__asm__ __volatile__("movl %%" #seg ",%0" : "=m" (where)) */
-
 static struct {
 	unsigned long	offset;
 	unsigned short	segment;
@@ -101,17 +94,16 @@ static struct desc_struct	bad_bios_desc = { 0, 0x00409200 };
  
 #ifdef CONFIG_SMP
 
-static unsigned long apm_save_cpus(void)
+static cpumask_t apm_save_cpus(void)
 {
-	unsigned long x = current->cpus_allowed;
+	cpumask_t x = current->cpus_allowed;
 	/* Some bioses don't like being called from CPU != 0 */
-	set_cpus_allowed(current, 1 << 0);
-	if (unlikely(smp_processor_id() != 0))
-		BUG();
+	set_cpus_allowed(current, cpumask_of_cpu(0));
+	BUG_ON(smp_processor_id() != 0);
 	return x;
 }
 
-static inline void apm_restore_cpus(unsigned long mask)
+static inline void apm_restore_cpus(cpumask_t mask)
 {
 	set_cpus_allowed(current, mask);
 }
@@ -122,7 +114,7 @@ static inline void apm_restore_cpus(unsigned long mask)
  *	No CPU lockdown needed on a uniprocessor
  */
  
-#define apm_save_cpus()	0
+#define apm_save_cpus()		(current->cpus_allowed)
 #define apm_restore_cpus(x)	(void)(x)
 
 #endif
@@ -175,6 +167,14 @@ static inline void apm_restore_cpus(unsigned long mask)
 #	define APM_DO_RESTORE_SEGS
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
+#define get_gdt_table()	(cpu_gdt_table[get_cpu()])
+#define put_gdt_table()	put_cpu()
+#else
+#define get_gdt_table()	(get_cpu_ptr(cpu_gdt_table))
+#define put_gdt_table()	put_cpu_ptr(cpu_gdt_table)
+#endif
+
 /**
  *	apm_bios_call	-	Make an APM BIOS 32bit call
  *	@func: APM function to execute
@@ -200,15 +200,14 @@ static u8 apm_bios_call(u32 func, u32 ebx_in, u32 ecx_in,
 {
 	APM_DECL_SEGS
 	unsigned long		flags;
-	unsigned long		cpus;
-	int			cpu;
-	struct desc_struct	save_desc_40;
+	cpumask_t		cpus;
+	struct desc_struct	save_desc_40, *desc40_ptr;
 
 	cpus = apm_save_cpus();
 	
-	cpu = get_cpu();
-	save_desc_40 = cpu_gdt_table[cpu][0x40 / 8];
-	cpu_gdt_table[cpu][0x40 / 8] = bad_bios_desc;
+	desc40_ptr = get_gdt_table() + (0x40/8);
+	save_desc_40 = *desc40_ptr;
+	*desc40_ptr = bad_bios_desc;
 
 	local_save_flags(flags);
 	APM_DO_CLI;
@@ -231,8 +230,8 @@ static u8 apm_bios_call(u32 func, u32 ebx_in, u32 ecx_in,
 		: "memory", "cc");
 	APM_DO_RESTORE_SEGS;
 	local_irq_restore(flags);
-	cpu_gdt_table[cpu][0x40 / 8] = save_desc_40;
-	put_cpu();
+	*desc40_ptr = save_desc_40;
+	put_gdt_table();
 	apm_restore_cpus(cpus);
 	
 	return *eax & 0xff;
@@ -257,16 +256,14 @@ static u8 apm_bios_call_simple(u32 func, u32 ebx_in, u32 ecx_in, u32 *eax)
 	u8			error;
 	APM_DECL_SEGS
 	unsigned long		flags;
-	unsigned long		cpus;
-	int			cpu;
-	struct desc_struct	save_desc_40;
-
+	cpumask_t		cpus;
+	struct desc_struct	save_desc_40, *desc40_ptr;
 
 	cpus = apm_save_cpus();
 	
-	cpu = get_cpu();
-	save_desc_40 = cpu_gdt_table[cpu][0x40 / 8];
-	cpu_gdt_table[cpu][0x40 / 8] = bad_bios_desc;
+	desc40_ptr = get_gdt_table() + (0x40/8);
+	save_desc_40 = *desc40_ptr;
+	*desc40_ptr = bad_bios_desc;
 
 	local_save_flags(flags);
 	APM_DO_CLI;
@@ -293,8 +290,8 @@ static u8 apm_bios_call_simple(u32 func, u32 ebx_in, u32 ecx_in, u32 *eax)
 	}
 	APM_DO_RESTORE_SEGS;
 	local_irq_restore(flags);
-	cpu_gdt_table[smp_processor_id()][0x40 / 8] = save_desc_40;
-	put_cpu();
+	*desc40_ptr = save_desc_40;
+	put_gdt_table();
 	apm_restore_cpus(cpus);
 	return error;
 }
