@@ -2,8 +2,8 @@
 /*********************************************************************
  *                
  * Filename:      smapi_call.h
- * Description:   header file for the super I/O device
- * Author:        Thomas Hood <jdthood@mail.com>
+ * Description:   header file for the smapi call code
+ * Author:        Thomas Hood
  * Created:       19 July 1999 
  *
  * Please report bugs to the author ASAP.
@@ -26,11 +26,11 @@
  *     
  ********************************************************************/
 
-#if !defined(__SMAPI_CALL_H__)
+#ifndef __SMAPI_CALL_H__
 #define __SMAPI_CALL_H__
 
 /****** declarations ******/
-int SMAPI_BIOS_do( smapi_inparm_t * , smapi_outparm_t * );
+asmlinkage int SMAPI_BIOS_do( smapi_inparm_t * , smapi_outparm_t * );
 extern dword _dwAddressSmapiBios;
 
 #endif
